#define _MULTI_THREADED
#define NUM_THREADS 40

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <syslog.h>

#include "ees_eics.h"
#include "xacml.h"
#include "xacml_server.h"
#include "eef_library.h"
#define PORT 1234 

int process_xacml(int);
void signal_handler(int);

/********************************************************************
function:       process_xacml
description:
                This function is the start of the service. It initializes the XACML library
                and will put the library in listen mode. In this case that I/O layer is hardwired to be secured
                via my I/O handler function implementation, that brings you the SSL layer. And in effect, HTTPS.

paramters:
      int listenport    :    The service will listen on this TCP port number
********************************************************************/
int process_xacml (int listenport)
{
    int done = 0;
    xacml_server_t server;
    unsigned short port = listenport;
    /* xacml_response_t response; */


    xacml_init(); /* if you look at the code, you'll see this function is empty */
    if(xacml_server_init(&server, eef_xacml_authorize, NULL) == XACML_RESULT_SUCCESS){
      /* Set server object properties to listen to give en port */
      xacml_server_set_port(server, listenport);
      xacml_server_get_port(server, &port);
      EEF_log(LOG_INFO, "SCAS ready... listening on port %hu.\n", port);


      /* Set the I/O handler object */
      /* xacml_server_set_io_descriptor(server, &xacml_io_ssl_descriptor); */

      /* start server and block further execution */
      xacml_server_start(server);
      while (!done) {
        pause();
        EEF_log(LOG_INFO, "SCAS service stop.\n");
      }
    }
    /* SIGTERM received */
    xacml_server_destroy(server);

    fflush(stdout);
    fflush(stderr);

    /* Perfect ending */
    return 0;
}

void signal_handler(int sig){
  switch(sig){
    case SIGHUP:
      EEF_log(LOG_NOTICE, "Got SIGHUP - restart \n");
      EEF_term();
      break;
    case SIGTERM:
      EEF_log(LOG_NOTICE, "Got SIGTERM - shutting down\n");
      AOS_Clean();
      EEF_term();
      exit(0);
      break;
  }
}

int main (int argc, char* argv[]){
  char* config_file;                      /* config file path */
  aos_context_t*   _context;
  aos_attribute_t *_uid, *_gid;

  char* _uid_str = "1234";
  char* _gid_str = "1234";


  if(argc > 1){
    config_file = argv[1];
  } else {
    config_file = "examples/example_1.pdl";
  }

#if !ENABLE_DEBUG
  EEF_daemonize();
  /*daemon(0,0);*/
#endif

  signal(SIGHUP, signal_handler); /* catch hangup signal */
  signal(SIGTERM, signal_handler); /* catch kill signal */

  /* reuse this part when implementing SIGHUP */
  EEF_log(LOG_INFO, "Parsing config file %s\n", config_file);
	if(EEF_init(config_file, EEF_log) == EES_FAILURE){
    EEF_log(LOG_ERR, "Failed to parse config file (%s), exiting...", config_file);
  } else {
    /*make_socket();*/
    /* fill aos */
    _uid = createAttribute();
    _gid = createAttribute();

    setAttributeId(_uid, "posix-uid");
    setAttributeValue(_uid, _uid_str, strlen(_uid_str));
    setAttributeId(_gid, "posix-gid");
    setAttributeValue(_gid, _gid_str, strlen(_gid_str));

    if((_context = createContext(OBLIGATION)) != NULL){
      setContextObligationId(_context, "uidgid");

      addAttribute(_context, _uid);
      addAttribute(_context, _gid);

      addContext(_context);
    }
    aos_dump_argslist();
    aos_start_threading();
    process_xacml(PORT);
    /* start threading */
    /*for(t=0; t < NUM_THREADS; t++){*/
    /*printf("Creating thread %li!\n", t);*/
    /*rc = pthread_create(&threads[t], NULL, (void*)(void*) EEF_Run, (void*)(t));*/
    /*rc = pthread_join(threads[t], &status);*/

    /**//*Fire up SAML2-XACML2 */
    /**//*process_xacml(PORT);*/

    /*}*/
  }
  EEF_log(LOG_INFO, "EEF terminated!\n");

  aos_dump_argslist();
  /*pthread_exit(NULL);*/

  return 0;
}


