%option noyywrap

%{
#include "pdl.h"
#include "pdl_yacc.h"
#include "eef_log.h"
#include <string.h>
unsigned int lineno = 1;

#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) \
    EEF_log(LOG_ERR, "Fatal parsing error: %s", (msg));
#endif

/*static int no_valid_rule=0;*/
%}

ws           [\t ]+
nl           \n
port         [1-9][0-9]*
term         [a-zA-Z_.0-9]+
label        {term}:
trans        "->"
comment      #.*\n
tokens       [\~=\|]
pvar         "path"
portvar      "port"
logvar       "log"
path         \.*\/[^\/ \t\n]*
pdlstr       \"[^\"\n]*[\"\n]


%x linecom path pathcom mq
%%

{ws}         ;
{pvar}       { return PVAR; }
{portvar}    { return PORTVAR; }
{port}	     { if ((yylval.record = (record_t*)malloc(sizeof(record_t))))
                 if (((yylval.record)->string = (char*) strdup(yytext))) {
                   (yylval.record)->lineno = lineno;
                   return PORT;
                 }
               EEF_log(LOG_ERR, "out of memory.");
               return 0;
             }
{logvar}     { return LOGVAR; }
{term}       { if ((yylval.record = (record_t*)malloc(sizeof(record_t))))
                 if (((yylval.record)->string = (char*) strdup(yytext))) {
                   (yylval.record)->lineno = lineno;
                   return TERM;
                 }
               EEF_log(LOG_ERR, "out of memory.");
               return 0;
             }
{label}      { yytext[yyleng-1] = '\0';   /* remove trailing ':' */

               /*  Check if this policy rule is allowed or not. */
               /*
               if (allowed_policy_rule(yytext)) {
                 allow_rules(TRUE);
               } else {
                 */
                 /*  This label should not be added, however normal parsing
                  *  must continue.  */
                 /* allow_rules(FALSE);*/
               /* } */
               if ((yylval.record = (record_t*)malloc(sizeof(record_t))))
                 if (((yylval.record)->string = (char *) strdup(yytext))) {
                   (yylval.record)->lineno = lineno;
                   return LABEL;
                 }
               EEF_log(LOG_ERR, "out of memory.");
               return 0; 
             }
{path}       { if ((yylval.record = (record_t *)malloc(sizeof(record_t))))
                 if (((yylval.record)->string = strdup(yytext))) {
                   (yylval.record)->lineno = lineno;
                   BEGIN path;
                   return PATH;
                 }
               EEF_log(LOG_ERR, "out of memory.");
               return 0; 
             }
<path>{path} { if ((yylval.record = (record_t *)malloc(sizeof(record_t))))
                 if (((yylval.record)->string = strdup(yytext))) {
                   (yylval.record)->lineno = lineno;
                   return PATH;
                 }
               EEF_log(LOG_ERR, "out of memory.");
               return 0; 
             }
<path>#      { BEGIN pathcom; }
<path>[ \t]*  ;
<path>\n     { BEGIN INITIAL; yyless(0); return '\n'; }
<path>.      { EEF_log(LOG_ERR, "unrecognized tokens in path name."); }
<pathcom>.   ;
<pathcom>\n  { BEGIN INITIAL; yyless(0); return '\n'; }
{nl}         { ++lineno;          }
{tokens}     { return yytext[0];  }
{trans}      { return TRANS;      }
{comment}    { ++lineno; BEGIN INITIAL ; }
{pdlstr}     { if ((yylval.record = (record_t*)malloc(sizeof(record_t)))) {
                 if (yytext[yyleng-2] == '\\') {
                   if (((yylval.record)->string = strdup(yytext+1))) {
                     (yylval.record)->string[yyleng-2] = '\0';
                     (yylval.record)->string[yyleng-3] = '\"';
                     (yylval.record)->lineno = lineno;
                     yyless(yyleng-1);  /*  make the next string start with "  */
                     BEGIN mq;
                     return STRING;
                   }
                 } else {
                   if (yytext[yyleng-1] == '\"') {
                     if (((yylval.record)->string = strdup(yytext+1))) {
                       (yylval.record)->string[yyleng-2] = '\0';
                       (yylval.record)->lineno = lineno;
                       return STRING;
                     }
                   }
                 }
                 EEF_log(LOG_ERR, "out of memory.");
                 return 0;
               } else
                 EEF_log(LOG_ERR, "string contains a newline.");
             }
<mq>{pdlstr} { if ((yylval.record = (record_t*)malloc(sizeof(record_t)))) {
                 if (yytext[yyleng-2] == '\\') {
                   if (((yylval.record)->string = strdup(yytext+1))) {
                     (yylval.record)->string[yyleng-2] = '\0';
                     (yylval.record)->string[yyleng-3] = '\"';
                     (yylval.record)->lineno = lineno;
                     yyless(yyleng-1);  /*  make the next string start with "  */
                     BEGIN INITIAL;
                     return STRING;
                   }
                 } else {
                   EEF_log(LOG_ERR, "Unbalanced quotes.");
                   return 0;
                 }
               }
               EEF_log(LOG_ERR, "out of memory.");
               return 0;
             }
.            { EEF_log(LOG_WARNING, "unrecognized token '%c'.", yytext[0]);
               return yytext[0];
             }
%%

#if HAVE_FLEX

void delete_lex_buffer(void)
{
  yy_delete_buffer(YY_CURRENT_BUFFER);
  yy_init = 1;
}

#endif
